﻿Type=Class
Version=2.5
@EndOfDesignText@
#Region Library Attributes
	#Event: Click
	#Event: LongClick
#End Region

'Class Name: PinchZoomAndMove
'Author: Dominex
'Version: 1.42
'B4A Version Used: 2.50
'Last Modified: 11/02/2013
'-------------------------
'Class module
Private Sub Class_Globals
	Private G As Gestures
	Private Parent As Panel
	Private Touch As List
	Private X0,Y0,X1,Y1,Hypotenuse,Product,posTouchX,posTouchY,OrigW,OrigH As Int
	Private vZoom,vZoomStart,vZoomMin,vZoomMax,vIncrease As Float
	Private vCanMove,SingleClick,vBringToFront,vCanZoom,BorderLimit As Boolean
	Private MovingTouch As Int = 10dip
	Private vTag,vModule As Object
	Private TimerLC As Timer
	Private obj As View
	Private vEventName As String
	Private TimePress As Long
	Type TouchData (X As Int,Y As Int,Tag As Object)
	Private vTouchData As TouchData
End Sub

'Initialize the View.
'<i>Target</i> - View on which to apply the PinchZoomAndMove.
'<i>NameEvent</i> - name of the event.
'<i>ZoomMin</i> - minimum Zoom (eg. 0.5 = 50%).
'<i>ZoomMax</i> - maximum Zoom (eg. 3 = 300%).
'<i>Increase</i> - factor of increase / decrease during the Pinch Zoom.
'<i>Tag</i> - is the Tags View.
'<i>BringToFront</i> - if True active the BringToFront.
'<i>Module</i> - must be Me.
Public Sub Initialize (Target As View,NameEvent As String,Tag As Object,ZoomMin As Float,ZoomMax As Float,Increase As Float,BringToFront As Boolean,Module As Object)
	Touch.Initialize
	obj = Target
	vModule = Module
	OrigH = obj.Height
	OrigW = obj.Width
	vZoom = OrigW
	vZoomStart = vZoom
	vZoomMin = vZoom*ZoomMin
	vZoomMax = vZoom*ZoomMax
	vIncrease = Increase
	vCanMove = True
	vCanZoom = True
	vEventName = NameEvent
	vTag = Tag
	vTouchData.Initialize
	vBringToFront = BringToFront
	TimerLC.Initialize("TimerLC",500)
	G.SetOnTouchListener(obj,"Touch_Gestures")
End Sub

'If it is true, the View will not come out from the edges of its parent.
Public Sub SetLimitBorder (Option As Boolean)
	BorderLimit = Option
	Parent = GetParent(obj)
	If obj.Width > Parent.Width Then
		obj.Width = Parent.Width
		obj.Height = OrigH/OrigW*obj.Width
	Else If obj.Height > Parent.Height Then
		obj.Height = Parent.Height
		obj.Width = OrigW/OrigH*obj.Height
	End If
	obj.Left = Min(Max(obj.Left,0),Parent.Width-obj.Width)
	obj.Top = Min(Max(obj.Top,0),Parent.Height-obj.Height)
End Sub

'Return if the View can exit or not from the edges of its parent.
Public Sub GetIfLimitBorder As Boolean
	Return BorderLimit
End Sub

'Sets whether the View may zoom or not.
'<i>Option</i> - True if the View can Zoom.
Public Sub SetIfCanZoom (Option As Boolean)
	vCanZoom = Option
End Sub

'Returns if the View can Zoom.
Public Sub GetIfCanZoom As Boolean
	Return vCanZoom
End Sub

'Sets whether the View may translare or not.
'<i>Option</i> - True if the View can translate.
Public Sub SetIfCanMove (Option As Boolean)
	vCanMove = Option
End Sub

'Returns if the View can translate.
Public Sub GetIfCanMove As Boolean
	Return vCanMove
End Sub

'Returns the current Zoom in percentage (eg. 100 = 100%).
Public Sub GetZoom As Float
	Return 100/vZoomStart*vZoom
End Sub

'Sets the zoom.
'<i>Zoom</i> - Zoom in percentage (eg. 100 = 100% = initial size of the View).
Public Sub SetZoom (Zoom As Float)
	ZoomView (Zoom/100*vZoomStart,obj.Width/2,obj.Height/2)
	vZoom = vZoomStart/100*Zoom
End Sub

Private Sub GetParent(v As View) As View
    Dim r As Reflector
    r.Target = v
    Return r.RunMethod("getParent")
End Sub

Private Sub TimerLC_Tick
	Dim margine = 5dip As Int
	TimerLC.Enabled = False
	If SubExists(vModule,vEventName&"_LongClick") AND SingleClick Then
		vTouchData.Tag = vTag
		CallSub2(vModule,vEventName&"_LongClick",vTouchData)
		SingleClick = False
	End If
End Sub

Private Sub Touch_Gestures(View As Object, PointerID As Int, Action As Int, X As Float, Y As Float) As Boolean
	Dim conta,xTemp,yTemp,Hypo,NewZoom As Int
	Select Action
		Case G.ACTION_DOWN
			Touch.Add(PointerID)
			SingleClick = True
			If vBringToFront Then obj.BringToFront
			TimePress = DateTime.Now
			'---Stores the position of View---
			X0 = X+obj.Left
			Y0 = Y+obj.Top
			X1 = obj.Left
			Y1 = obj.Top
			'---Stores the position for LongClick---
			vTouchData.X = X
			vTouchData.Y = Y
			TimerLC.Enabled = True
		Case G.ACTION_POINTER_DOWN
			Touch.Add(PointerID)
			If Touch.Size = 2 Then 
				xTemp = G.GetX(Touch.Get(0))-G.GetX(Touch.Get(1))
				yTemp = G.GetY(Touch.Get(0))-G.GetY(Touch.Get(1))
				Hypotenuse = Sqrt(Power(xTemp,2)+Power(yTemp,2))/Density
			End If
			SingleClick = False
		Case G.ACTION_MOVE
			'---clears the first slight movement to the first pressure---
			If DateTime.Now-TimePress < 100 Then
				X0 = X+obj.Left
				Y0 = Y+obj.Top
				X1 = obj.Left
				Y1 = obj.Top
			End If
			If Touch.Size = 1 Then
				'---Move View---
				If vCanMove Then
					xTemp = X1+G.GetX(Touch.Get(0))+obj.Left-X0
					yTemp = Y1+G.GetY(Touch.Get(0))+obj.Top-Y0
					If BorderLimit Then
						xTemp = Min(Max(xTemp,0),Parent.Width-obj.Width)
						yTemp = Min(Max(yTemp,0),Parent.Height-obj.Height)
					End If
					obj.Left = xTemp
					obj.Top = yTemp
				End If
				If Abs(vTouchData.X-X) > MovingTouch OR Abs(vTouchData.Y-Y) > MovingTouch Then TimerLC.Enabled = False
			Else If Touch.Size = 2 AND vCanZoom Then
				'---Pinch Zoom---
				xTemp = G.GetX(Touch.Get(0))-G.GetX(Touch.Get(1))
				yTemp = G.GetY(Touch.Get(0))-G.GetY(Touch.Get(1))
				Hypo = Sqrt(Power(xTemp,2)+Power(yTemp,2))/Density
				Product = (Hypo-Hypotenuse)*vIncrease
				NewZoom = vZoom+Product
				NewZoom = Min(NewZoom,vZoomMax)
				NewZoom = Max(NewZoom,vZoomMin)
				posTouchX = Min(G.GetX(Touch.Get(0)),G.GetX(Touch.Get(1)))+Abs(xTemp/2)
				posTouchY = Min(G.GetY(Touch.Get(0)),G.GetY(Touch.Get(1)))+Abs(yTemp/2)
				ZoomView (NewZoom,posTouchX,posTouchY) 'Show the new zoom
			End If
		Case G.ACTION_POINTER_UP
			'---Removes touches are no longer present on the screen---
			For conta = 0 To Touch.Size-1
				If Touch.Get(conta) = PointerID Then
					Touch.RemoveAt(conta)
					Exit
				End If
			Next
			If Touch.Size = 1 Then
				X0 = G.GetX(Touch.Get(0))+obj.Left
				Y0 = G.GetY(Touch.Get(0))+obj.Top
				X1 = obj.Left
				Y1 = obj.Top
				'---Save the size of View---
				vZoom = vZoom+Product
				vZoom = Min(vZoom,vZoomMax)
				vZoom = Max(vZoom,vZoomMin)
			End If
		Case G.ACTION_UP
			Touch.Clear
			TimerLC.Enabled = False
			If Abs(X+obj.Left-X0) < MovingTouch AND Abs(Y+obj.Top-Y0) < MovingTouch AND SingleClick Then
				If SubExists(vModule,vEventName&"_Click") Then
					vTouchData.Tag = vTag
					CallSub2(vModule,vEventName&"_Click",vTouchData)
				End If
			End If
	End Select
	Return True
End Sub

Private Sub ZoomView (NewZoom As Int,posX As Int,posY As Int)
	Dim Width,Height As Int
	Width = NewZoom
	Height = OrigH/OrigW*Width
	If BorderLimit Then
		If Width > Parent.Width Then
			Width = Parent.Width
			Height = OrigH/OrigW*Width
		Else If Height > Parent.Height Then
			Height = Parent.Height
			Width = OrigW/OrigH*Height
		End If
	End If
	Dim Left,Top As Int
	Left = obj.Left+posX-(posX/obj.Width*Width)
	Top = obj.Top+posY-(posY/obj.Height*Height)
	If BorderLimit Then
		Left = Min(Max(Left,0),Parent.Width-obj.Width)
		Top = Min(Max(Top,0),Parent.Height-obj.Height)
	End If
	obj.SetLayout(Left,Top,Width,Height)
End Sub
